import { useState } from 'react';
import {
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  Button,
  Box,
  Typography,
  TextField,
} from '@mui/material';
import { Palette } from '@mui/icons-material';
import { useThemeStore } from '../stores/themeStore';

interface ColorPickerProps {
  open: boolean;
  onClose: () => void;
}

const predefinedColors = [
  // Purples (3 shades as requested)
  { name: 'Deep Purple', value: '#673ab7' },
  { name: 'Medium Purple', value: '#9c27b0' },
  { name: 'Light Purple', value: '#e1bee7' },

  // Blues
  { name: 'Deep Blue', value: '#1976d2' },
  { name: 'Light Blue', value: '#03dac6' },
  { name: 'Indigo', value: '#3f51b5' },

  // Greens
  { name: 'Forest Green', value: '#2e7d32' },
  { name: 'Mint Green', value: '#4caf50' },
  { name: 'Teal', value: '#009688' },

  // Reds/Oranges
  { name: 'Deep Red', value: '#d32f2f' },
  { name: 'Coral', value: '#ff5722' },
  { name: 'Amber', value: '#ff9800' },

  // Grays
  { name: 'Charcoal', value: '#424242' },
  { name: 'Slate', value: '#607d8b' },
  { name: 'Warm Gray', value: '#795548' },
];

export function ColorPicker({ open, onClose }: ColorPickerProps) {
  const { primaryColor, setPrimaryColor } = useThemeStore();
  const [customColor, setCustomColor] = useState(primaryColor);

  const handleColorSelect = (color: string) => {
    setPrimaryColor(color);
    onClose();
  };

  const handleCustomColorSubmit = () => {
    if (customColor && /^#[0-9A-F]{6}$/i.test(customColor)) {
      setPrimaryColor(customColor);
      onClose();
    }
  };

  return (
    <Dialog open={open} onClose={onClose} maxWidth="sm" fullWidth>
      <DialogTitle>
        <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
          <Palette />
          <Typography>Choose Primary Color</Typography>
        </Box>
      </DialogTitle>
      <DialogContent>
        <Box sx={{ mb: 3 }}>
          <Typography variant="h6" sx={{ mb: 2 }}>
            Predefined Colors
          </Typography>
          <Box sx={{ display: 'grid', gridTemplateColumns: 'repeat(5, 1fr)', gap: 1 }}>
            {predefinedColors.map((color) => (
              <Box
                key={color.value}
                sx={{
                  display: 'flex',
                  flexDirection: 'column',
                  alignItems: 'center',
                  cursor: 'pointer',
                  p: 1,
                  borderRadius: 1,
                  border: primaryColor === color.value ? 2 : 1,
                  borderColor: primaryColor === color.value ? 'primary.main' : 'divider',
                  '&:hover': {
                    bgcolor: 'action.hover',
                  },
                }}
                onClick={() => handleColorSelect(color.value)}
              >
                <Box
                  sx={{
                    width: 40,
                    height: 40,
                    borderRadius: '50%',
                    bgcolor: color.value,
                    border: 1,
                    borderColor: 'divider',
                    mb: 0.5,
                  }}
                />
                <Typography variant="caption" sx={{ textAlign: 'center', fontSize: '0.7rem' }}>
                  {color.name}
                </Typography>
              </Box>
            ))}
          </Box>
        </Box>

        <Box>
          <Typography variant="h6" sx={{ mb: 2 }}>
            Custom Color
          </Typography>
          <Box sx={{ display: 'flex', gap: 1, alignItems: 'center' }}>
            <TextField
              value={customColor}
              onChange={(e) => setCustomColor(e.target.value)}
              placeholder="#000000"
              size="small"
              sx={{ flex: 1 }}
            />
            <Box
              sx={{
                width: 40,
                height: 40,
                borderRadius: '50%',
                bgcolor: customColor,
                border: 1,
                borderColor: 'divider',
              }}
            />
            <Button
              variant="contained"
              onClick={handleCustomColorSubmit}
              disabled={!customColor || !/^#[0-9A-F]{6}$/i.test(customColor)}
            >
              Apply
            </Button>
          </Box>
        </Box>
      </DialogContent>
      <DialogActions>
        <Button onClick={onClose}>Cancel</Button>
      </DialogActions>
    </Dialog>
  );
}